#! /usr/bin/env python
#
# The purpose of this python program is to scan through valgrind runs
# looking for memory leaks or bad exit statuses.
#
# It takes a list of executables to run through valgrind.

from os import popen
from sys import stdout
from re import compile
from time import sleep

suppressionFiles = [ ]
failedRuns = []
leakyRuns = []
suppressionMatch = compile(r"^--suppressions=")
definiteLeakedBytes = 0
definiteLeakedBlocks = 0
possibleLeakedBytes = 0
possibleLeakedBlocks = 0

def runValgrind(executable):
    global failedRuns, definiteLeakedBytes, definiteLeakedBlocks
    global possibleLeakedBytes, possibleLeakedBlocks, leakyRuns
    errors = compile(r"^==\d+==\s+ERROR SUMMARY:\s+(\d+)\s+errors from\s+(\d+)\s+contexts")
    definite = compile(r"^==\d+==\s+definitely lost:\s+(\d+)\s+bytes in\s+(\d+)\s+blocks.")
    possible = compile(r"^==\d+==\s+possibly lost:\s+(\d+)\s+bytes in\s+(\d+)\s+blocks.")
    if ((executable[0] != '/') and
        (executable[0] != '.')):
        executable = "./" + executable
    print "Running", executable
    run = popen("valgrind --leak-check=yes " +
                " ".join(suppressionFiles) + 
               " --show-reachable=yes ./" +
                executable + " 2>&1")
    line = run.readline()
    while (line):
        check = errors.match(line)
        if check and eval(check.group(1)):
            failedRuns = failedRuns + [ executable + " " + check.group(1) +
                                        " errors in " + check.group(2) +
                                        " contexts" ]
        check = definite.match(line)
        if check:
            leakedBytes = eval(check.group(1))
            leakedBlocks = eval(check.group(2))
            definiteLeakedBytes = definiteLeakedBytes + leakedBytes
            definiteLeakedBlocks = definiteLeakedBlocks + leakedBlocks
            if leakedBytes:
                leakyRuns = leakyRuns + [ executable + ' definitely leaked ' +
                                          str(leakedBytes) + " bytes in " +
                                          str(leakedBlocks) + " blocks." ]
        check = possible.match(line)
        if check:
            leakedBytes = eval(check.group(1))
            leakedBlocks = eval(check.group(2))
            possibleLeakedBytes = possibleLeakedBytes + leakedBytes
            possibleLeakedBlocks = possibleLeakedBlocks + leakedBlocks
            if leakedBytes:
                leakyRuns = leakyRuns + [ executable + ' possibly leaked ' +
                                          str(leakedBytes) + " bytes in " +
                                          str(leakedBlocks) + " blocks (possible)." ]
        stdout.write(line)
        line = run.readline()
    sleep(0.2)
    exitstatus = run.close();
    if exitstatus:
        print "Program", executable, "exited with status", exitstatus
        failedRuns = failedRuns + [ executable + " " + str(exitstatus)]

def summarizeRun():
    print "=============================="
    print "SUMMARY OF VALGRIND RUNS"
    print "=============================="
    
    print len(failedRuns), "failed or had errors"
    for i in failedRuns:
        print ' ', i
    print definiteLeakedBytes, "bytes leaked in", definiteLeakedBlocks, "blocks"
    print possibleLeakedBytes, "bytes leaked in", possibleLeakedBlocks, "blocks"
    for i in leakyRuns:
        print ' ', i


if __name__ == '__main__':
    from sys import argv
    for i in argv[1:]:
        if (suppressionMatch.match(i)):
            suppressionFiles = suppressionFiles + [ i ]
        else:
            runValgrind(i)
    summarizeRun()



#!/usr/local/bin/python
#
# File:        HelloDriver.py
# Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
# Revision:    @(#) $Revision: 6171 $
# Date:        $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
# Description: Simple CCA Hello World Python client 
#

import decaf.Framework
import gov.cca.ports.GoPort

if __name__ == '__main__':
  dec = decaf.Framework.Framework()
  
  server = dec.createInstance( "HelloServerInstance","HelloServer.Component", None )
  client = dec.createInstance( "HelloClientInstance","HelloClient.Component", None )
  dec.connect(client, "HelloServer", server, "HelloServer")

  port = dec.lookupPort(client, "GoPort")
  go = gov.cca.ports.GoPort.GoPort(port)
  go.go()

  dec.destroyInstance(server,0.0)
  dec.destroyInstance(client,0.0)



#
# File:          __init__.py
# Symbol:        sidl.rmi-v0.9.17
# Symbol Type:   package
# Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
# Release:       $Name:  $
# Revision:      @(#) $Id: $
# Description:   package initialization code
# 
# Copyright (c) 2000-2002, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by the Components Team <components@llnl.gov>
# All rights reserved.
# 
# This file is part of Babel. For more information, see
# http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
# for Our Notice and the LICENSE file for the GNU Lesser General Public
# License.
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by
# the Free Software Foundation) version 2.1 dated February 1999.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
# conditions of the GNU Lesser General Public License for more details.
# 
# You should have recieved a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# 
# WARNING: Automatically generated; changes will be lost
# 
#


""" 
This package contains necessary interfaces for RMI protocols to
hook into Babel, plus a Protocol Factory class.  The intention is
that authors of new protocols will create classes that implement
InstanceHandle, Invocation and Response (they could even have one
object that implements all three interfaces).
"""

__all__ = [
   "BindException",
   "Call",
   "ConnectException",
   "ConnectRegistry",
   "InstanceHandle",
   "InstanceRegistry",
   "Invocation",
   "MalformedURLException",
   "NetworkException",
   "NoRouteToHostException",
   "NoServerException",
   "ObjectDoesNotExistException",
   "ProtocolException",
   "ProtocolFactory",
   "Response",
   "Return",
   "ServerInfo",
   "ServerRegistry",
   "Ticket",
   "TicketBook",
   "TimeOutException",
   "UnexpectedCloseException",
   "UnknownHostException" ]

try:
  from pkgutil import extend_path
  __path__ = extend_path(__path__, __name__)
except: # ignore all exceptions
  pass



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

"""
Module in the Gantlet Testing Framework
"""

import os.path
import string

def check_vpath_build( config, dirname ):
    """
    Checks if VPATH mode is enabled.  If so,
    return bindir, which is a relative path from
    the srcdir back to the build directory.

    NOTE: that dirname should already have srcdir
    prepended to it.
    """

    if ( config.srcdir == os.curdir ):
        return os.curdir

    if ( config.verbose > 0 ):
        print "vpath build detected..."

    fullpath = os.getcwd().split( os.sep );
    testdirs = dirname.split( os.sep );
    bindirs = []
    n_dirs_up = 0
    # now build bindirs by iterating over testdirs in reverse
    for i in range(len(testdirs)-1,-1,-1):
        if ( testdirs[i] == os.pardir ):
            n_dirs_up += 1
        elif (testdirs[i] == os.curdir ):
            print "WARNING: %s found in result of path_to_list." % os.curdir
        else:
            bindirs.append( os.pardir )

    # the last n_dirs_up go into bindir
    # which takes us back down the directory srcdir goes up.
    bindir = string.join( bindirs + fullpath[-n_dirs_up:], os.sep )

    if ( config.verbose > 1 ):
        print "  current working directory (absolute) = " + \
              string.join( fullpath, os.sep )
        print "  directory where tests (scripts) are run (relative) = " + \
              string.join( testdirs, os.sep )
        print "  directory for scripts to find compiled binaries = " + \
              bindir

    return bindir



#
# File:          HeapSort_Impl.py
# Symbol:        sort.HeapSort-v0.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class sort.HeapSort in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""Heap sort
"""

# DO-NOT-DELETE splicer.begin(_initial)
# DO-NOT-DELETE splicer.end(_initial)

import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sort.Comparator
import sort.Container
import sort.Counter
import sort.HeapSort
import sort.SortingAlgorithm
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
def remakeHeap(elem, comp, cmp, swp, last, first):
  half = (last >> 1) - 1
  while (first <= half):
    child = first + first + 1
    if ((child + 1) < last):
      cmp.inc()
      if (elem.compare(child, child+1, comp) < 0): child = child +1
    cmp.inc()
    if (elem.compare(first, child, comp) >= 0): return
    swp.inc()
    elem.swap(first, child)
    first = child
# DO-NOT-DELETE splicer.end(_before_type)

class HeapSort:
  """\
Heap sort
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = sort.HeapSort.HeapSort(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def sort(self, elems, comp):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # sort.Container elems
    # sort.Comparator comp
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Sort elements using Heap Sort.
"""
# DO-NOT-DELETE splicer.begin(sort)
    size = elems.getLength()
    cmp = self.__IORself.getCompareCounter()
    swp = self.__IORself.getSwapCounter()
    for i in xrange((size >> 1) - 1, -1, -1):
      remakeHeap(elems, comp, cmp, swp, size, i)
    i = size - 1
    while (i > 0):
      swp.inc()
      elems.swap(0, i)
      remakeHeap(elems, comp, cmp, swp, i, 0)
      i = i - 1
# DO-NOT-DELETE splicer.end(sort)

  def getName(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

    """\
Return heap sort.
"""
# DO-NOT-DELETE splicer.begin(getName)
    return "Heap sort"
# DO-NOT-DELETE splicer.end(getName)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# File:          FibException_Impl.py
# Symbol:        ExceptionTest.FibException-v1.0
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class ExceptionTest.FibException in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""This exception is a base class for the Fibonacci exceptions that are
thrown if the value is too large or the recursion depth is too deep.
"""

# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import ExceptionTest.FibException
import sidl.BaseClass
import sidl.BaseException
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.SIDLException
import sidl.io.Deserializer
import sidl.io.Serializable
import sidl.io.Serializer
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
# Put your code here...
# DO-NOT-DELETE splicer.end(_before_type)

class FibException:
  """\
This exception is a base class for the Fibonacci exceptions that are
thrown if the value is too large or the recursion depth is too deep.
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = ExceptionTest.FibException.FibException(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

import os
import time
import smtplib
import string
import gantlet
from gantlet.events import *
from gantlet.xml_display import xml_display
import getopt
import sys
import re
import socket

class misc_obj:
    pass

std_header = 'From: %s\nTo: %s\nDate: %s\nSubject: %s\nPrecedence: bulk\nMIME-Version: 1.0\nContent-Type: text/xml; charset=iso-8859-1 \n'

def rfc822time():    
    if time.daylight:
        offset = -time.altzone / 3600
    else:
        offset = -time.timezone / 3600            
    date = time.strftime("%a, %d %b %Y %H:%M:%S %%+03d00 (%Z)") % offset
    return date

class email_display ( xml_display ):

    def __init__( self, scoreboard, email_to, mailserver ):
        if scoreboard:
            file = os.tmpfile( ) # some temporary file.
            xml_display.__init__( self, scoreboard, file )
        else:
            self.file = None
        self.email_to = email_to
        self.email_from = None
        self.mailserver = mailserver
        self.closefile = 0 # don't close file at xml_display.display_final()
        self.verbose = None
            
    def display_final( self, scoreboard ):
        """ This overrides the method registered to receive suite_finish events"""
        xml_display.display_final( self, scoreboard )
        # now email the XML file
        self.email_file( scoreboard )
        return

    def email_file( self, scoreboard ):
        """Email a non-broken Session

        Called either in the process of receiving a suite_finish event
        or from wrap_n_send_xml()
        """
        
        self.email_from = scoreboard.whoami + "@" + scoreboard.hostname
        date = rfc822time()
        if scoreboard.tot_tests_failed + scoreboard.tot_tests_broken > 0:
            result = 'FAIL'
        elif scoreboard.tot_tests_xfailed > 0:
            result = 'XFAIL'
        else:
            result = 'PASS'            
        subject = '<gantlet-xml package="%s" result="%s" profile="%s" session="%s" >' % \
                  ( scoreboard.package, result, scoreboard.profile, scoreboard.session)
        header = std_header % (self.email_from, self.email_to, date, subject )
        self.email_file_via_SMTP( header )

    def email_file_via_SMTP(  self, header, f=None ):
        if f == None:
            file = self.file
        file.seek(0) # rewind file
        text = file.readlines() #read lines
        text = header + string.join( text, "" )
        if f!=None:
            file.close()
        
        # now send the email
        server = smtplib.SMTP( self.mailserver )
        if self.verbose:
            print "Ready to send email..."
            print header
            print "Content: %d bytes " % len( text )
        failed = server.sendmail( self.email_from, self.email_to, text )
        server.quit()
        if self.verbose:
            if failed: # SMTP may raise exceptions, but let them pass
                print 'Failed Email Recipients:', failed
            else:
                print 'Email Sent'

    def fake_scoreboard_from_file(self):
        """Creates a bogus scoreboard object so that send_email() can create the email headers

        Used in wrap_n_send_xml(), which implies being run by the commandline, instead of
        a real testing session.
        """
        _have_attribute = re.compile(r"<attribute key=\"([^\"]*)\" value=\"([^\"]*)\"")
        _end_header = re.compile("<head/>")
        o = misc_obj
        self.file.seek(0)        
        for i in range(50):
            line = self.file.readline()
            b = _end_header.search(line)
            if b:
                break
            m = _have_attribute.search(line)
            if m:
                setattr(o,m.group(1),m.group(2))
        setattr(o,"tot_tests_failed", int(o.failed_tests) )
        setattr(o,"tot_tests_broken", int(o.broken_tests) )
        setattr(o,"tot_tests_xfailed", int(o.xfailed_tests) )
        setattr(o,"tot_tests_passed", int(o.passed_tests) )
        return o

    def email_broken(self,package,profile,session):
        date = rfc822time()
        subject = '<gantlet-xml package="%s" result="%s" profile="%s" session="%s">' % \
                  ( package, 'BROKEN', profile, session)
        header = std_header %  (self.email_from, self.email_to, date, subject )
        self.email_file_via_SMTP( header )

help_string="""
USAGE:
   ( -h ) | ( [-v] -e<> -m<> (-f<> | -b<> -p<> -k<> -n<>) )
 -h   | --help                       Display this message
 -v   | --verbose                    Verbose
 -e<> | --email=<>                   <>=string, send email to this address
 -m<> | --mailserver=<>              <>=string, domain name of SMTP server
 -x<> | --xmlsrc=<>                  <>=file, wrap up the XML report generated by GANTLET
 -b<> | --broken=<>                  <>=logfile.  at least send why it failed.
 -k<> | --package=<>                 <>=string, name of the package being tested
 -p<> | --profile=<>                 <>=string, configuration info (e.g. ix86-linux-gcc)
 -n<> | --name=<>                    <>=string, name of the test suit (e.g. unit, regression, all)
"""

def wrap_n_send_xml():
    verbose = None
    xmlsrc = None
    broken = None
    mailserver = None
    package = ''
    profile = ''
    name = ''
    shortopts="hvb:e:m:x:k:p:n:"
    longopts=["help","verbose","email=","xmlsrc=","mailserver=","broken"]
    (opts,args) = getopt.getopt( sys.argv[1:], shortopts, longopts )
    for (o,v) in opts:
        if o in ("-h", "--help"):
            print help_string
            sys.exit(0)
        elif o in ("-v", "--verbose"):
            verbose = 1
        elif o in ("-e", "--email"):
            email = v
        elif o in ("-m", "--mailserver"):
            mailserver=v
        elif o in ("-x", "--xmlsrc"):
            xmlsrc = v
        elif o in ("-b", "--broken"):
            broken = 1
            xmlsrc = v
        elif o in ("-k", "--package"):
            package = v
        elif o in ("-p", "--profile"):
            profile = v
        elif o in ("-n", "--name"):
            name = v

    e = email_display( None, email, mailserver  )
    e.verbose = verbose
    e.file = open(xmlsrc,'r')
    if broken:
        e.email_from =  "%s@%s" %  (os.getenv('USER','gantlet'),
                                    socket.gethostbyaddr(socket.gethostname())[0])
        e.email_broken(package,profile,name)
        pass # email contents of file with broken header
    elif xmlsrc:
        s = e.fake_scoreboard_from_file()
        e.email_file(s)
    
if __name__ == '__main__':
    wrap_n_send_xml()
    



#!/usr/local/bin/python
#
# File:        helloclient
# Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
# Revision:    @(#) $Revision: 6171 $
# Date:        $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
# Description: Simple Hello World Python client 
#

#import sidl.BaseClass
import Hello.World

if __name__ == '__main__':
  h = Hello.World.World ()
  print h.getMsg ()



#
# File:          CallType.py
# Symbol:        sidlx.rmi.CallType-v0.1
# Symbol Type:   enumeration
# Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
# Description:   define enumeration constants
# 
# WARNING: Automatically generated; changes will be lost
#


"""There are 3 basic types of calls on the server side, CREATE, EXEC, and     \
  SERIAL.
This enumeration defines them.  
"""

CREATE = 0

EXEC = 1

SERIAL = 2

ERROR = 3




#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

import gantlet.events
from gantlet.result_base import result_base
from gantlet.tparser_base import tparser_base

class unknown_tparser(result_base, tparser_base):

    def __init__( self, invocation, scoreboard): 
        tparser_base.__init__( self, invocation, scoreboard ) 
        self.errmsgs = invocation.errmsgs
	result_base.__init__(self, invocation.symbolic_name, scoreboard, "unknown")

    def get_type(self):
	return "unknown_tparser"

    def runtest(self):
        self.number = self.scoreboard.get_id()
        self.scoreboard.event['test_begin'].publish( gantlet.events.test_begin_event, self)
        for (msg,canon) in self.errmsgs:
            self.broken(msg,canon)
        self.scoreboard.event['test_end'].publish( gantlet.events.test_end_event, self )
        



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

import os
import gantlet.events

class Part:
    pass

class result_base:
    """
    result_base:  A Mixin class for all test results

    Has the following member data:
       name             - (string)  
       n_parts          - (int) number of parts in the test
       n_parts_passed   - (int) number of parts passed in the test
       n_parts_xfailed  - (int) number of parts having expected failures
       n_parts_failed   - (int) number of parts having failed in the test
       is_broken        - (0 or 1) if the test was broken
       exit_status      - (int or None) exit status of the program
       errmsg           - (string) accumulated error messages
       canon            - (string) list of failed parts
       part             - list of parts
       number           - (int)
       result           - (string) 'PASS','XFAIL','FAIL',or 'BROKEN'
       type             - (string) type of test
       
    """
    def __init__( self, raw_name, scoreboard, type):
        """
        result_base( \"symbolic test name\", \"file\",
                     \"entire test string\", \"type\" )
        """
        self.scoreboard               = scoreboard
        self.name                     = raw_name
        self.raw_name                 = raw_name
        self.n_parts                  = 0
	self.n_parts_passed           = 0
        self.n_parts_xfailed          = 0
        self.n_parts_failed           = 0
        self.is_broken                = 0
        self.exit_status              = None
        self.errmsg                   = ''
        self.canon                    = ''
        self.part                     = []        
        self.number                   = -1
        self.type                     = type
                
    def broken( self, msg, canon ):
        """
        Set the test as broken
        """
        self.errmsg += "ERROR: " + msg
        self.is_broken = 1
        self.result = 'BROKEN'
        if ( canon != "" ):
            self.canon += "[%s] " % canon
        self.scoreboard.event['test_error'].publish(
            gantlet.events.test_error_event, (self, msg, canon) )
        
    def warning( self, msg, canon ):
        """
        Associate a warning with the test, but do not break
        """
        self.errmsg = "WARNING: " + msg
        if ( canon != "" ):
            self.canon += "[%s] " % canon
        self.scoreboard.event['test_warning'].publish(
            gantlet.events.test_warning_event, (self, msg, canon) )

    #
    # The following methods beginning with PTC should be considered
    # protected methods of this class
    #
    def PTC_confirm_nparts( self ):
        """
        Double check the number of parts in the test

        PTC_ prefix implies the method is protected.
        """
        self.n_parts = len( self.part )
        if self.declared_nparts == -1:
            pass
        elif self.declared_nparts == len ( self.part ):
            pass
        elif self.declared_nparts > len(self.part):
            msg = "declared %d parts, but found only %d." % \
                  (self.declared_nparts, len(self.part))
            self.warning( msg , 'NPARTS high' ) 
        elif self.declared_nparts < len(self.part):
            msg = "declared %d parts, but found %d." % \
                  (self.declared_nparts, len(self.part))
            self.warning( msg , 'NPARTS low' ) 
                    
    def PTC_confirm_result( self ):
        """
        Confirm the declared result is the same as the computed one.

        The result of the test can be computed by the results of
        the constituent parts.

        PTC_ prefix implies the method is protected.
        """
        # 1. If we already know its broken, go no farther
        if self.is_broken:
            return

        # 2. tally up the results of all the parts
        n_pass = 0
        n_xfail = 0
        n_fail = 0
        for part in self.part:
            if part.result == 'PASS':
                n_pass = n_pass + 1
            elif part.result == 'XFAIL' or \
                 part.result == 'UNTESTED' or \
                 part.result == 'UNSUPPORTED':
                n_xfail = n_xfail + 1
            elif part.result == 'FAIL' or \
                 part.result == 'XPASS' or \
                 part.result == 'UNRESOLVED':
                n_fail = n_fail + 1

        # 3. confirm that all parts are accounted for
        if n_pass + n_xfail + n_fail != self.n_parts:
            msg = "only %d out of %d part results recognized" % \
                  n_pass + n_xfail + n_fail, self.n_parts
            self.broken( msg, 'state error')
            state = 'broken'
            return

        # 4. compute result
        if n_fail > 0:
            self.result = 'FAIL'
        elif n_xfail > 0:
            self.result = 'XFAIL'
        elif n_pass != 0:
            self.result = 'PASS'
        else:
            self.result = 'BROKEN'
        # 5. compare against declared result
        if self.result != self.declared_result:
            if self.declared_result == 'BROKEN':
                self.result = 'BROKEN'
            elif self.declared_result == 'FAIL' and not self.result == 'BROKEN':
                self.result = 'FAIL'
            elif self.declared_result == 'XFAIL' and self.result == 'PASS':
                self.result = 'XFAIL'
            msg = "declared result \"%s\" != computed result \"%s\"" % \
                (self.declared_result, self.result)
            self.warning( msg, 'declared/computed test result mismatch')
        # 6. save counts
        self.n_parts_passed = n_pass
        self.n_parts_xfailed = n_xfail
        self.n_parts_failed = n_fail

def _test():
    """
    This regression test needs to be redone with unittest
    """
    pass
    # t = test('autoexec.bat',r'C:\autoexec.bat',r'C:\autoexec.bat')
    # print t.__dict__
    # 
    # t = test('winhelp.exe',r'C:\WINNT\winhelp.exe',r'C:\WINNT\winhelp.exe')
    # print t.__dict__
    # 
    # t2 = test("`test 2`",'test2', 'test2')
    # print t2.__dict__
    # 
    # t2 = test('foo','test',"`PETF_NAME=foo test 2`")
    # print t2.__dict__


if __name__ == '__main__':
    _test()



#
# File:          StructTest_Impl.py
# Symbol:        s.StructTest-v1.0
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class s.StructTest in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


# DO-NOT-DELETE splicer.begin(_initial)
# Insert-Code-Here {_initial} ()
# DO-NOT-DELETE splicer.end(_initial)

import s.Combined
import s.Empty
import s.Hard
import s.Simple
import s.StructTest
import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
import s.Color
s_pkg = s

import sidlPyArrays
if sidlPyArrays.type == "numpy":
  from numpy import zeros, float32
elif sidlPyArrays.type == "numeric":
  import Numeric
  zeros = Numeric.zeros
  float32 = Numeric.Float32

def toFloat(d):
  tmp = zeros((1,), float32)
  try:
    tmp[0] = d
  except AttributeError:
    # don't die!
    tmp[0] = d
  return float(tmp[0])

def initSimple():
  s = s_pkg.Simple.Simple()
  s.d_bool = 1
  s.d_char = '3'
  s.d_dcomplex = 3.14+3.14j
  s.d_fcomplex = toFloat(3.1)+ toFloat(3.1) * 1j
  s.d_double = 3.14
  s.d_float = toFloat(3.1)
  s.d_int = 3
  s.d_long = 3
  s.d_opaque = None
  s.d_enum = s_pkg.Color.blue
  return s

def checkSimple(s):
  return (s.d_bool and (s.d_char == '3') and (s.d_dcomplex == 3.14+3.14j) and
          (s.d_double == 3.14) and
          (s.d_fcomplex == (toFloat(3.1) + toFloat(3.1) * 1j)) and
          (s.d_float == toFloat(3.1)) and (3 == s.d_int) and
          (3 == s.d_long) and (s.d_opaque == None) and
          (s_pkg.Color.blue == s.d_enum))

def invertSimple(s):
  s.d_bool = not s.d_bool
  if (s.d_char.islower()):
    s.d_char = s.d_char.upper()
  else:
    s.d_char =  s.d_char.lower()
  s.d_dcomplex = s.d_dcomplex.conjugate()
  s.d_fcomplex = s.d_fcomplex.conjugate()
  s.d_double = -s.d_double
  s.d_float = -s.d_float
  s.d_int = -s.d_int
  s.d_long = -s.d_long
  s.d_enum = s_pkg.Color.red
  return s

def initHard():
  h = s_pkg.Hard.Hard()
  h.d_string = "Three"
  h.d_object = sidl.BaseClass.BaseClass()
  h.d_interface = h.d_object
  h.d_array = [ 1.0, 2.0, 3.0 ]
  h.d_objectArray = [ sidl.BaseClass.BaseClass(),
                      sidl.BaseClass.BaseClass(),
                      sidl.BaseClass.BaseClass() ]
  return h

def checkHard(h):
  return (("Three" == h.d_string) and (None != h.d_object) and
          (None != h.d_interface) and h.d_object.isSame(h.d_interface) and
          (len(h.d_array) == 3) and (h.d_array[0] == 1.0) and
          (h.d_array[1] == 2.0) and (h.d_array[2] == 3.0) and
          (len(h.d_objectArray) == 3) and
          h.d_objectArray[0].isType("sidl.BaseClass") and
          h.d_objectArray[1].isType("sidl.BaseClass") and
          h.d_objectArray[2].isType("sidl.BaseClass"))

def invertHard(h):
  if ((None != h.d_string) and (len(h.d_string) > 0)):
    if (h.d_string[0].islower()):
      h.d_string = h.d_string[0].upper() + h.d_string[1:]
    else:
      h.d_string = h.d_string[0].lower() + h.d_string[1:]
  if ((None != h.d_object) and (None != h.d_interface)):
    if (h.d_object.isSame(h.d_interface)):
      h.d_interface = sidl.BaseClass.BaseClass()
    else:
      h.d_interface = h.d_object
  if (len(h.d_array) == 3):
    (h.d_array[0], h.d_array[2]) = (h.d_array[2], h.d_array[0])
  oa = h.d_objectArray
  if ((len(oa) == 3)):
    if (oa[1] != None):
      oa[1] = None
    else:
      oa[1] = sidl.BaseClass.BaseClass()
  h.d_objectArray = oa
  return h

def initCombined():
  c = s_pkg.Combined.Combined()
  c.d_simple = initSimple()
  c.d_hard = initHard()
  return c

def checkCombined(c):
  return checkSimple(c.d_simple) and checkHard(c.d_hard)

def invertCombined(c):
  invertSimple(c.d_simple)
  invertHard(c.d_hard)
  return c
# DO-NOT-DELETE splicer.end(_before_type)

class StructTest:

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = s.StructTest.StructTest(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def returnEmpty(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # s.Empty _return
    #

# DO-NOT-DELETE splicer.begin(returnEmpty)
    return s_pkg.Empty.Empty()
# DO-NOT-DELETE splicer.end(returnEmpty)

  def passinEmpty(self, s):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Empty s
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # bool _return
    #

# DO-NOT-DELETE splicer.begin(passinEmpty)
    return 1
# DO-NOT-DELETE splicer.end(passinEmpty)

  def passoutEmpty(self):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s)
    # bool _return
    # s.Empty s
    #

# DO-NOT-DELETE splicer.begin(passoutEmpty)
    return (1, s_pkg.Empty.Empty())
# DO-NOT-DELETE splicer.end(passoutEmpty)

  def passinoutEmpty(self, s):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Empty s
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s)
    # bool _return
    # s.Empty s
    #

# DO-NOT-DELETE splicer.begin(passinoutEmpty)
    return (1, s_pkg.Empty.Empty())
# DO-NOT-DELETE splicer.end(passinoutEmpty)

  def passeverywhereEmpty(self, s1, s3):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Empty s1
    # s.Empty s3
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s2, s3)
    # s.Empty _return
    # s.Empty s2
    # s.Empty s3
    #

# DO-NOT-DELETE splicer.begin(passeverywhereEmpty)
    return (s.Empty.Empty(), s.Empty.Empty(), s.Empty.Empty())
# DO-NOT-DELETE splicer.end(passeverywhereEmpty)

  def returnSimple(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # s.Simple _return
    #

# DO-NOT-DELETE splicer.begin(returnSimple)
    return initSimple()
# DO-NOT-DELETE splicer.end(returnSimple)

  def passinSimple(self, s):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Simple s
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # bool _return
    #

# DO-NOT-DELETE splicer.begin(passinSimple)
    return checkSimple(s)
# DO-NOT-DELETE splicer.end(passinSimple)

  def passoutSimple(self):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s)
    # bool _return
    # s.Simple s
    #

# DO-NOT-DELETE splicer.begin(passoutSimple)
    return (1, initSimple())
# DO-NOT-DELETE splicer.end(passoutSimple)

  def passinoutSimple(self, s):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Simple s
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s)
    # bool _return
    # s.Simple s
    #

# DO-NOT-DELETE splicer.begin(passinoutSimple)
    return (checkSimple(s), invertSimple(s))
# DO-NOT-DELETE splicer.end(passinoutSimple)

  def passeverywhereSimple(self, s1, s3):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Simple s1
    # s.Simple s3
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s2, s3)
    # s.Simple _return
    # s.Simple s2
    # s.Simple s3
    #

# DO-NOT-DELETE splicer.begin(passeverywhereSimple)
    if (checkSimple(s1) and checkSimple(s3)):
      s3 = invertSimple(s3)
    return (initSimple(), initSimple(), s3)
# DO-NOT-DELETE splicer.end(passeverywhereSimple)

  def returnHard(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # s.Hard _return
    #

# DO-NOT-DELETE splicer.begin(returnHard)
    return initHard()
# DO-NOT-DELETE splicer.end(returnHard)

  def passinHard(self, s):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Hard s
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # bool _return
    #

# DO-NOT-DELETE splicer.begin(passinHard)
    return checkHard(s)
# DO-NOT-DELETE splicer.end(passinHard)

  def passoutHard(self):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s)
    # bool _return
    # s.Hard s
    #

# DO-NOT-DELETE splicer.begin(passoutHard)
    return (1, initHard())
# DO-NOT-DELETE splicer.end(passoutHard)

  def passinoutHard(self, s):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Hard s
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s)
    # bool _return
    # s.Hard s
    #

# DO-NOT-DELETE splicer.begin(passinoutHard)
    return (checkHard(s), invertHard(s))
# DO-NOT-DELETE splicer.end(passinoutHard)

  def passeverywhereHard(self, s1, s3):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Hard s1
    # s.Hard s3
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s2, s3)
    # s.Hard _return
    # s.Hard s2
    # s.Hard s3
    #

# DO-NOT-DELETE splicer.begin(passeverywhereHard)
    if (checkHard(s1) and checkHard(s3)):
      s3 = invertHard(s3)
    return (initHard(), initHard(), s3)
# DO-NOT-DELETE splicer.end(passeverywhereHard)

  def returnCombined(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # s.Combined _return
    #

# DO-NOT-DELETE splicer.begin(returnCombined)
    return initCombined()
# DO-NOT-DELETE splicer.end(returnCombined)

  def passinCombined(self, s):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Combined s
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # bool _return
    #

# DO-NOT-DELETE splicer.begin(passinCombined)
    return checkCombined(s)
# DO-NOT-DELETE splicer.end(passinCombined)

  def passoutCombined(self):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s)
    # bool _return
    # s.Combined s
    #

# DO-NOT-DELETE splicer.begin(passoutCombined)
    return (1, initCombined())
# DO-NOT-DELETE splicer.end(passoutCombined)

  def passinoutCombined(self, s):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Combined s
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s)
    # bool _return
    # s.Combined s
    #

# DO-NOT-DELETE splicer.begin(passinoutCombined)
    return (checkCombined(s), invertCombined(s))
# DO-NOT-DELETE splicer.end(passinoutCombined)

  def passeverywhereCombined(self, s1, s3):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # s.Combined s1
    # s.Combined s3
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, s2, s3)
    # s.Combined _return
    # s.Combined s2
    # s.Combined s3
    #

# DO-NOT-DELETE splicer.begin(passeverywhereCombined)
    if (checkCombined(s1) and checkCombined(s3)):
      s3 = invertCombined(s3)
    return (initCombined(), initCombined(), s3)
# DO-NOT-DELETE splicer.end(passeverywhereCombined)

# DO-NOT-DELETE splicer.begin(_final)
# Insert-Code-Here {_final} ()
# DO-NOT-DELETE splicer.end(_final)



#
# File:          ArrayOps_Impl.py
# Symbol:        ArrayTest.ArrayOps-v1.3
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class ArrayTest.ArrayOps in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


# DO-NOT-DELETE splicer.begin(_initial)
import sidlPyArrays
if sidlPyArrays.type == "numpy":
 import numpy
 zeros = numpy.zeros
 float32 = numpy.float32
 ones = numpy.ones
 int32 = numpy.int32
 int64 = numpy.int64
 float64 = numpy.float64
 float32 = numpy.float32
 complex64 = numpy.complex64
 complex128 = numpy.complex128
 ArrayType = numpy.ndarray
 objectArray = numpy.object
 strArray = numpy.str
elif sidlPyArrays.type == "numeric":
 import Numeric
 int32 = Numeric.Int32
 int64 = Numeric.Int32
 float32 = Numeric.Float32
 float64 = Numeric.Float64
 strArray = Numeric.Character
 complex64 = Numeric.Complex32
 complex128 = Numeric.Complex
 zeros = Numeric.zeros
 ones = Numeric.ones
 ArrayType = Numeric.ArrayType
 objectArray = Numeric.PyObject
from types import NoneType

stringData = ( "I'd", "rather", "write", "programs", "to", "write",
               "programs", "than", "write", "programs." )
stringDataLen = len(stringData)
              

charData = "I'd rather write programs to write programs than write programs."
charDataLen = len(charData)

def savespace(o):
  try:
      o.savespace(1)
  except AttributeError:
      pass

def typecode(o):
  try:
      return o.typecode()
  except AttributeError:
      return o.dtype.char

def powTwo(i):
    result = 1.0
    if (i >= 0):
      while (i):
        result = result *2
        i = i - 1
    else:
      while (i):
        result = result * 0.5
        i = i + 1
    return result

def powTwoF(i):
  result = zeros((1,), float32)
  savespace(result)
  result[0] = powTwo(i)
  return result[0]

def isEven(num):
  return (num & 1) == 0

def isPrime(num):
  i = 3l
  while (i*i <= num):
    if ((num % i) == 0):
      return 0
    i = i + 1
  return 1

def nextPrime(prev):
  if (prev <= 1l):
    return 2l
  else:
    if (prev == 2l):
      return 3l
    else:
      prev = prev + 2l
      while (not isPrime(prev)):
        prev = prev + 2l
  return prev

def reverseArray(array, newArray):
  if (newArray):
    copy = zeros(array.shape, typecode(array))
    savespace(copy)
    if (typecode(array) == 'c'):
        for i in range(array.shape[0]):
            copy[-(i+1)] = str(array[i])
    else:
        for i in range(array.shape[0]):
            copy[-(i+1)] = array[i]
  else:
    if (typecode(array) == 'c'):
        for i in range(array.shape[0] >> 1):
            array[-(i+1)], array[i] =  str(array[i]), str(array[-(i+1)])
    else:
        for i in range(array.shape[0] >> 1):
            array[-(i+1)], array[i] =  array[i], array[-(i+1)]
    copy = array
  return 1, copy

def arrayValue(l):
  result = 1
  add = 1
  for i in l:
    result = result * (i + add)
    add = add + 1
  return result

#Gives the approprite numeric array typecode for the given sidl typecode.
def sidlTypeToNumType(tp):

    if (tp == 1 or tp == 7):
        return int32
    elif (tp == 8):
        return int64
    elif (tp > 8):
        return objectArray
    elif (tp == 2):
        return strArray
    elif (tp == 3):
        return complex128
    elif (tp == 4):
        return float64
    elif (tp == 5):
        return complex64
    elif (tp == 6):
        return float32
    else:
        return objectArray

# this function makes a guess at the sidl type we're talking about from the
# numeric area type code.  It will be wrong for bools, longs,
# opaques, and strings.
def numTypeToSidlType(tp):
    if (tp == 'i'):
        return 7
    elif (tp == 'O'):
        return 11
    elif (tp == 'c'):
        return 2
    elif (tp == 'D'):
        return 3
    elif (tp == 'd'):
        return 4
    elif (tp == 'F'):
        return 5
    elif (tp == 'f'):
        return 6
    elif (tp == 'S'):
        return 2
    elif (tp == 'q'):
        return 8
    elif (tp == 'l'):
        return 7
    else:
        return 11
    

def createArrayByType(tp,ashape):
    result = ones(ashape, sidlTypeToNumType(tp))
    savespace(result)
    return result

def locMatrixMultiply(a, b, res):
    if(a.shape[0] == res.shape[0] and
       a.shape[1] == b.shape[0] and
       b.shape[1] == res.shape[1]):
        for i in range(a.shape[0]):
            for k in range(b.shape[1]):
                temp = 0;
                for j in range(a.shape[1]):
                    temp += (a[i,j] * b[j,k])
                res[i,k]=temp;
    return                
      
# DO-NOT-DELETE splicer.end(_initial)

import ArrayTest.ArrayOps
import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_static)
# Put your code here...
# DO-NOT-DELETE splicer.end(_before_static)

def checkBool(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<bool> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

  """\
Return <code>true</code> iff the even elements are true and
the odd elements are false.
"""
# DO-NOT-DELETE splicer.begin(checkBool)
  if (isinstance(a,ArrayType) and len(a.shape) == 1):
    for i in range(a.shape[0]):
      if (a[i] != isEven(i)):
        return 0
    return 1
  return 0
# DO-NOT-DELETE splicer.end(checkBool)

def checkChar(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<char> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkChar)
  if (isinstance(a,ArrayType) and len(a.shape) == 1):
    for i in range(a.shape[0]):
      if (a[i] != charData[i % charDataLen]):
        return 0
    return 1
  return 0
# DO-NOT-DELETE splicer.end(checkChar)

def checkInt(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkInt)
  if (isinstance(a,ArrayType) and (len(a.shape) == 1)):
    prime = nextPrime(0)
    for i in range(a.shape[0]):
      if (a[i] != prime):
        return 0
      prime = nextPrime(prime)
    return 1
  return 0
# DO-NOT-DELETE splicer.end(checkInt)

def checkLong(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<long> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkLong)
  if (isinstance(a,ArrayType) and (len(a.shape) == 1)):
    prime = nextPrime(0)
    for i in range(a.shape[0]):
      if (a[i] != prime):
        return 0
      prime = nextPrime(prime)
    return 1
  return 0
# DO-NOT-DELETE splicer.end(checkLong)

def checkString(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<string> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkString)
  if (isinstance(a,ArrayType) and len(a.shape) == 1):
    for i in range(a.shape[0]):
      if (a[i] != stringData[i % stringDataLen]):
        return 0
    return 1
  return 0
# DO-NOT-DELETE splicer.end(checkString)

def checkDouble(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkDouble)
  if (isinstance(a,ArrayType) and len(a.shape) == 1):
    for i in range(a.shape[0]):
      if (a[i] != powTwo(-i)):
        return 0
    return 1
  return 0
# DO-NOT-DELETE splicer.end(checkDouble)

def checkFloat(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<float> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkFloat)
  if (isinstance(a,ArrayType) and len(a.shape) == 1):
    for i in range(a.shape[0]):
      if (a[i] != powTwoF(-i)):
        return 0
    return 1
  return 0
# DO-NOT-DELETE splicer.end(checkFloat)

def checkFcomplex(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<fcomplex> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkFcomplex)
  if (isinstance(a,ArrayType) and len(a.shape) == 1):
    for i in range(a.shape[0]):
      if (not ((a[i].real == powTwoF(i)) and (a[i].imag == powTwoF(-i)))):
        return 0
    return 1
  return 0
# DO-NOT-DELETE splicer.end(checkFcomplex)

def checkDcomplex(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<dcomplex> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkDcomplex)
  if (isinstance(a,ArrayType) and len(a.shape) == 1):
    for i in range(a.shape[0]):
      if (not ((a[i].real == powTwo(i)) and (a[i].imag == powTwo(-i)))):
        return 0
    return 1
  return 0
# DO-NOT-DELETE splicer.end(checkDcomplex)

def check2Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,2> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check2Int)
  if (isinstance(a,ArrayType) and len(a.shape) == 2):
    for i in range(a.shape[0]):
      for j in range(a.shape[1]):
        if (a[i,j] != powTwo(abs(i-j))):
          return 0
    return 1
  return 0
# DO-NOT-DELETE splicer.end(check2Int)

def check2Double(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double,2> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check2Double)
  if (isinstance(a,ArrayType) and (len(a.shape) == 2)):
    for i in range(a.shape[0]):
      for j in range(a.shape[1]):
        if (a[i,j] != powTwo(i-j)):
            return 0
  else:
    return 0
  return 1
# DO-NOT-DELETE splicer.end(check2Double)

def check2Float(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<float,2> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check2Float)
  if (isinstance(a,ArrayType) and (len(a.shape) == 2)):
    for i in range(a.shape[0]):
      for j in range(a.shape[1]):
        if (a[i,j] != powTwoF(i-j)):
            return 0
  else:
    return 0
  return 1
# DO-NOT-DELETE splicer.end(check2Float)

def check2Fcomplex(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<fcomplex,2> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check2Fcomplex)
  if (isinstance(a,ArrayType) and (len(a.shape) == 2)):
      for i in range(a.shape[0]):
          for j in range(a.shape[1]):
              if ((a[i,j].real != powTwoF(i)) or
                  (a[i,j].imag != powTwoF(-j))):
                return 0
  else:
    return 0
  return 1
# DO-NOT-DELETE splicer.end(check2Fcomplex)

def check2Dcomplex(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<dcomplex,2> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check2Dcomplex)
  if (isinstance(a,ArrayType) and (len(a.shape) == 2)):
      for i in range(a.shape[0]):
          for j in range(a.shape[1]):
              if ((a[i,j].real != powTwo(i)) or
                  (a[i,j].imag != powTwo(-j))):
                return 0
  else:
    return 0
  return 1
# DO-NOT-DELETE splicer.end(check2Dcomplex)

def check3Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,3> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check3Int)
  if (isinstance(a,ArrayType) and (len(a.shape) == 3)):
    for i in range(a.shape[0]):
      for j in range(a.shape[1]):
        for k in range(a.shape[2]):
          if (a[i,j,k] != arrayValue((i,j,k))):
            return 0
  else:
    return 0
  return 1
# DO-NOT-DELETE splicer.end(check3Int)

def check4Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,4> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check4Int)
  if (isinstance(a,ArrayType) and (len(a.shape) == 4)):
    for i in range(a.shape[0]):
      for j in range(a.shape[1]):
        for k in range(a.shape[2]):
          for l in range(a.shape[3]):
            if (a[i,j,k,l] != arrayValue((i,j,k,l))):
              return 0
  else:
    return 0
  return 1
# DO-NOT-DELETE splicer.end(check4Int)

def check5Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,5> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check5Int)
  if (isinstance(a,ArrayType) and (len(a.shape) == 5)):
    for i in range(a.shape[0]):
      for j in range(a.shape[1]):
        for k in range(a.shape[2]):
          for l in range(a.shape[3]):
            for m in range(a.shape[4]):
              if (a[i,j,k,l,m] != arrayValue((i,j,k,l,m))):
                return 0
  else:
    return 0
  return 1
# DO-NOT-DELETE splicer.end(check5Int)

def check6Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,6> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check6Int)
  if (isinstance(a,ArrayType) and (len(a.shape) == 6)):
    for i in range(a.shape[0]):
      for j in range(a.shape[1]):
        for k in range(a.shape[2]):
          for l in range(a.shape[3]):
            for m in range(a.shape[4]):
              for n in range(a.shape[5]):  
                if (a[i,j,k,l,m,n] != arrayValue((i,j,k,l,m,n))):
                  return 0           
  else:
    return 0
  return 1
# DO-NOT-DELETE splicer.end(check6Int)

def check7Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,7> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check7Int)
  if (isinstance(a,ArrayType) and (len(a.shape) == 7)):
    for i in range(a.shape[0]):
      for j in range(a.shape[1]):
        for k in range(a.shape[2]):
          for l in range(a.shape[3]):
            for m in range(a.shape[4]):
              for n in range(a.shape[5]):
                for o in range(a.shape[5]):
                  if (a[i,j,k,l,m,n,o] != arrayValue((i,j,k,l,m,n,o))):
                    return 0  
  else:
    return 0
  return 1
# DO-NOT-DELETE splicer.end(check7Int)

def check2String(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<string,2> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(check2String)
  c = 0
  if (isinstance(a,ArrayType) and len(a.shape) == 2):
      for i in range(a.shape[0]):
          for j in range(a.shape[1]):
              if (a[i,j] != stringData[c % stringDataLen]):
                  return 0
              ++c
      return 1
  return 0
# DO-NOT-DELETE splicer.end(check2String)

def checkObject(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<ArrayTest.ArrayOps> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # int _return
  #

# DO-NOT-DELETE splicer.begin(checkObject)
  count = 0
  if (isinstance(a,ArrayType) and (len(a.shape) == 1)):
    for i in range(a.shape[0]):
      obj = a[i]
      if (obj and ArrayTest.ArrayOps.ArrayOps(obj)):
        count = count + 1
  return count
# DO-NOT-DELETE splicer.end(checkObject)

def reverseBool(a, newArray):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<bool> a
  # bool newArray
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, a)
  # bool _return
  # array<bool> a
  #

# DO-NOT-DELETE splicer.begin(reverseBool)
  return reverseArray(a, newArray)
# DO-NOT-DELETE splicer.end(reverseBool)

def reverseChar(a, newArray):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<char> a
  # bool newArray
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, a)
  # bool _return
  # array<char> a
  #

# DO-NOT-DELETE splicer.begin(reverseChar)
  return reverseArray(a, newArray)
# DO-NOT-DELETE splicer.end(reverseChar)

def reverseInt(a, newArray):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int> a
  # bool newArray
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, a)
  # bool _return
  # array<int> a
  #

# DO-NOT-DELETE splicer.begin(reverseInt)
  return reverseArray(a, newArray)
# DO-NOT-DELETE splicer.end(reverseInt)

def reverseLong(a, newArray):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<long> a
  # bool newArray
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, a)
  # bool _return
  # array<long> a
  #

# DO-NOT-DELETE splicer.begin(reverseLong)
  return reverseArray(a, newArray)
# DO-NOT-DELETE splicer.end(reverseLong)

def reverseString(a, newArray):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<string> a
  # bool newArray
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, a)
  # bool _return
  # array<string> a
  #

# DO-NOT-DELETE splicer.begin(reverseString)
  return reverseArray(a, newArray)
# DO-NOT-DELETE splicer.end(reverseString)

def reverseDouble(a, newArray):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> a
  # bool newArray
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, a)
  # bool _return
  # array<double> a
  #

# DO-NOT-DELETE splicer.begin(reverseDouble)
  return reverseArray(a, newArray)
# DO-NOT-DELETE splicer.end(reverseDouble)

def reverseFloat(a, newArray):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<float> a
  # bool newArray
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, a)
  # bool _return
  # array<float> a
  #

# DO-NOT-DELETE splicer.begin(reverseFloat)
  return reverseArray(a, newArray)
# DO-NOT-DELETE splicer.end(reverseFloat)

def reverseFcomplex(a, newArray):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<fcomplex> a
  # bool newArray
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, a)
  # bool _return
  # array<fcomplex> a
  #

# DO-NOT-DELETE splicer.begin(reverseFcomplex)
  return reverseArray(a, newArray)
# DO-NOT-DELETE splicer.end(reverseFcomplex)

def reverseDcomplex(a, newArray):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<dcomplex> a
  # bool newArray
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, a)
  # bool _return
  # array<dcomplex> a
  #

# DO-NOT-DELETE splicer.begin(reverseDcomplex)
  return reverseArray(a, newArray)
# DO-NOT-DELETE splicer.end(reverseDcomplex)

def createBool(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<bool> _return
  #

# DO-NOT-DELETE splicer.begin(createBool)
  result = None
  if (len >= 0):
      result = zeros((len, ), int32)
      savespace(result)
      for i in range(len):
          result[i] = isEven(i)
  return result
# DO-NOT-DELETE splicer.end(createBool)

def createChar(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<char> _return
  #

# DO-NOT-DELETE splicer.begin(createChar)
  result = None
  if (len >= 0):
      result = zeros((len, ), 'c')
      savespace(result)
      for i in range(len):
          result[i] = charData[i % charDataLen]
  return result
# DO-NOT-DELETE splicer.end(createChar)

def createInt(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int> _return
  #

# DO-NOT-DELETE splicer.begin(createInt)
  result = None
  if (len >= 0):
      result = zeros((len, ), int32)
      savespace(result)
      prime = 0
      for i in range(len):
          prime = nextPrime(prime)
          result[i] = prime
  return result
# DO-NOT-DELETE splicer.end(createInt)

def createLong(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<long> _return
  #

# DO-NOT-DELETE splicer.begin(createLong)
  result = None
  if (len >= 0):
      result = zeros((len, ), int32)
      savespace(result)
      prime = 0
      for i in range(len):
          prime = nextPrime(prime)
          result[i] = prime
  return result
# DO-NOT-DELETE splicer.end(createLong)

def createString(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<string> _return
  #

# DO-NOT-DELETE splicer.begin(createString)
  result = None
  if (len >= 0):
      result = zeros((len, ), objectArray)
      savespace(result)
      for i in range(len):
          result[i] = stringData[i % stringDataLen]
  return result
# DO-NOT-DELETE splicer.end(createString)

def createDouble(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double> _return
  #

# DO-NOT-DELETE splicer.begin(createDouble)
  result = None
  if (len >= 0):
      result = zeros((len, ), float64)
      savespace(result)
      for i in range(len):
          result[i] = powTwo(-i)
  return result
# DO-NOT-DELETE splicer.end(createDouble)

def createFloat(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<float> _return
  #

# DO-NOT-DELETE splicer.begin(createFloat)
  result = None
  if (len >= 0):
      result = zeros((len, ), float32)
      savespace(result)
      for i in range(len):
          result[i] = powTwo(-i)
  return result
# DO-NOT-DELETE splicer.end(createFloat)

def createFcomplex(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<fcomplex> _return
  #

# DO-NOT-DELETE splicer.begin(createFcomplex)
  result = None
  if (len >= 0):
      result = zeros((len,), complex64)
      savespace(result)
      for i in range(result.shape[0]):
          result[i] = powTwo(i) + powTwo(-i)*1j
  return result
# DO-NOT-DELETE splicer.end(createFcomplex)

def createDcomplex(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<dcomplex> _return
  #

# DO-NOT-DELETE splicer.begin(createDcomplex)
  result = None
  if (len >= 0):
      result = zeros((len,), complex128)
      savespace(result)
      for i in range(result.shape[0]):
          result[i] = powTwo(i) + powTwo(-i)*1j
  return result
# DO-NOT-DELETE splicer.end(createDcomplex)

def createObject(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<ArrayTest.ArrayOps> _return
  #

# DO-NOT-DELETE splicer.begin(createObject)
  result = None
  if (len >= 0):
      result = zeros((len, ), objectArray)
      savespace(result)
      for i in range(len):
          result[i] = ArrayTest.ArrayOps.ArrayOps()
  return result
# DO-NOT-DELETE splicer.end(createObject)

def create2Int(d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,2> _return
  #

# DO-NOT-DELETE splicer.begin(create2Int)
  result = None
  if (d1 >= 0 and d2 >= 0):
      result = zeros((d1, d2, ), int32)
      savespace(result)
      for i in range(d1):
          for j in range(d2):
              result[i,j] = powTwo(abs(i-j))
  return result
# DO-NOT-DELETE splicer.end(create2Int)

def create2Double(d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double,2> _return
  #

# DO-NOT-DELETE splicer.begin(create2Double)
  result = None
  if (d1 >= 0 and d2 >= 0):
      result = zeros((d1, d2, ), float64)
      savespace(result)
      for i in range(d1):
          for j in range(d2):
              result[i,j] = powTwo(i-j)
  return result
# DO-NOT-DELETE splicer.end(create2Double)

def create2Float(d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<float,2> _return
  #

# DO-NOT-DELETE splicer.begin(create2Float)
  result = None
  if (d1 >= 0 and d2 >= 0):
      result = zeros((d1, d2, ), float32)
      savespace(result)
      for i in range(d1):
          for j in range(d2):
              result[i,j] = powTwoF(i-j)
  return result
# DO-NOT-DELETE splicer.end(create2Float)

def create2Dcomplex(d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<dcomplex,2> _return
  #

# DO-NOT-DELETE splicer.begin(create2Dcomplex)
  result = None
  if (d1 >= 0 and d2 >= 0):
      result = zeros((d1, d2, ), complex128)
      savespace(result)
      for i in range(d1):
          for j in range(d2):
              result[i,j] = complex(powTwo(i), powTwo(-j))
  return result
# DO-NOT-DELETE splicer.end(create2Dcomplex)

def create2Fcomplex(d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<fcomplex,2> _return
  #

# DO-NOT-DELETE splicer.begin(create2Fcomplex)
  result = None
  if (d1 >= 0 and d2 >= 0):
      result = zeros((d1, d2, ), complex64)
      savespace(result)
      for i in range(d1):
          for j in range(d2):
              result[i,j] = complex(powTwoF(i), powTwoF(-j))
  return result
# DO-NOT-DELETE splicer.end(create2Fcomplex)

def create2String(d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<string,2> _return
  #

# DO-NOT-DELETE splicer.begin(create2String)
  result = None
  c = 0
  if (d1 >= 0 and d2 >= 0):
      result = zeros((d1,d2, ), objectArray)
      savespace(result)
      for i in range(d1):
          for j in range(d2):
              result[i,j] = stringData[c % stringDataLen]
              ++c
  return result
# DO-NOT-DELETE splicer.end(create2String)

def create3Int():
  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,3> _return
  #

# DO-NOT-DELETE splicer.begin(create3Int)
  result = zeros((3, 3, 2, ), int32)
  savespace(result)
  for i in range(3):
    for j in range(3):
      for k in range(2):
        result[i,j,k] = arrayValue((i,j,k))
  return result
# DO-NOT-DELETE splicer.end(create3Int)

def create4Int():
  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,4> _return
  #

# DO-NOT-DELETE splicer.begin(create4Int)
  result = zeros((3, 3, 2, 2, ), int32)
  savespace(result)
  for i in range(3):
    for j in range(3):
      for k in range(2):
        for l in range(2):
          result[i,j,k,l] = arrayValue((i,j,k,l))
  return result
# DO-NOT-DELETE splicer.end(create4Int)

def create5Int():
  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,5> _return
  #

# DO-NOT-DELETE splicer.begin(create5Int)
  result = zeros((3, 3, 2, 2, 2, ), int32)
  savespace(result)
  for i in range(3):
    for j in range(3):
      for k in range(2):
        for l in range(2):
          for m in range(2):  
            result[i,j,k,l,m] = arrayValue((i,j,k,l,m))
  return result
# DO-NOT-DELETE splicer.end(create5Int)

def create6Int():
  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,6> _return
  #

# DO-NOT-DELETE splicer.begin(create6Int)
  result = zeros((3, 3, 2, 2, 2, 2, ), int32)
  savespace(result)
  for i in range(3):
    for j in range(3):
      for k in range(2):
        for l in range(2):
          for m in range(2):
            for n in range(2):
              result[i,j,k,l,m,n] = arrayValue((i,j,k,l,m,n))
  return result
# DO-NOT-DELETE splicer.end(create6Int)

def create7Int():
  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,7> _return
  #

# DO-NOT-DELETE splicer.begin(create7Int)
  result = zeros((3, 3, 2, 2, 2, 2, 2, ), int32)
  savespace(result)
  for i in range(3):
    for j in range(3):
      for k in range(2):
        for l in range(2):
          for m in range(2):
            for n in range(2):
              for o in range(2):
                result[i,j,k,l,m,n,o] = arrayValue((i,j,k,l,m,n,o))
  return result
# DO-NOT-DELETE splicer.end(create7Int)

def makeBool(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<bool> a
  #

# DO-NOT-DELETE splicer.begin(makeBool)
  return ArrayTest.ArrayOps.createBool(len)
# DO-NOT-DELETE splicer.end(makeBool)

def makeChar(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<char> a
  #

# DO-NOT-DELETE splicer.begin(makeChar)
  return ArrayTest.ArrayOps.createChar(len)
# DO-NOT-DELETE splicer.end(makeChar)

def makeInt(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int> a
  #

# DO-NOT-DELETE splicer.begin(makeInt)
  return ArrayTest.ArrayOps.createInt(len)
# DO-NOT-DELETE splicer.end(makeInt)

def makeLong(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<long> a
  #

# DO-NOT-DELETE splicer.begin(makeLong)
  return ArrayTest.ArrayOps.createLong(len)
# DO-NOT-DELETE splicer.end(makeLong)

def makeString(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<string> a
  #

# DO-NOT-DELETE splicer.begin(makeString)
  return ArrayTest.ArrayOps.createString(len)
# DO-NOT-DELETE splicer.end(makeString)

def makeDouble(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double> a
  #

# DO-NOT-DELETE splicer.begin(makeDouble)
  return ArrayTest.ArrayOps.createDouble(len)
# DO-NOT-DELETE splicer.end(makeDouble)

def makeFloat(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<float> a
  #

# DO-NOT-DELETE splicer.begin(makeFloat)
  return ArrayTest.ArrayOps.createFloat(len)
# DO-NOT-DELETE splicer.end(makeFloat)

def makeFcomplex(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<fcomplex> a
  #

# DO-NOT-DELETE splicer.begin(makeFcomplex)
  return ArrayTest.ArrayOps.createFcomplex(len)
# DO-NOT-DELETE splicer.end(makeFcomplex)

def makeDcomplex(len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<dcomplex> a
  #

# DO-NOT-DELETE splicer.begin(makeDcomplex)
  return ArrayTest.ArrayOps.createDcomplex(len)
# DO-NOT-DELETE splicer.end(makeDcomplex)

def makeInOutBool(a, len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<bool> a
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<bool> a
  #

# DO-NOT-DELETE splicer.begin(makeInOutBool)
  return ArrayTest.ArrayOps.createBool(len)
# DO-NOT-DELETE splicer.end(makeInOutBool)

def makeInOutChar(a, len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<char> a
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<char> a
  #

# DO-NOT-DELETE splicer.begin(makeInOutChar)
  return ArrayTest.ArrayOps.createChar(len)
# DO-NOT-DELETE splicer.end(makeInOutChar)

def makeInOutInt(a, len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int> a
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int> a
  #

# DO-NOT-DELETE splicer.begin(makeInOutInt)
  return ArrayTest.ArrayOps.createInt(len)
# DO-NOT-DELETE splicer.end(makeInOutInt)

def makeInOutLong(a, len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<long> a
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<long> a
  #

# DO-NOT-DELETE splicer.begin(makeInOutLong)
  return ArrayTest.ArrayOps.createLong(len)
# DO-NOT-DELETE splicer.end(makeInOutLong)

def makeInOutString(a, len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<string> a
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<string> a
  #

# DO-NOT-DELETE splicer.begin(makeInOutString)
  return ArrayTest.ArrayOps.createString(len)
# DO-NOT-DELETE splicer.end(makeInOutString)

def makeInOutDouble(a, len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> a
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double> a
  #

# DO-NOT-DELETE splicer.begin(makeInOutDouble)
  return ArrayTest.ArrayOps.createDouble(len)
# DO-NOT-DELETE splicer.end(makeInOutDouble)

def makeInOutFloat(a, len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<float> a
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<float> a
  #

# DO-NOT-DELETE splicer.begin(makeInOutFloat)
  return ArrayTest.ArrayOps.createFloat(len)
# DO-NOT-DELETE splicer.end(makeInOutFloat)

def makeInOutDcomplex(a, len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<dcomplex> a
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<dcomplex> a
  #

# DO-NOT-DELETE splicer.begin(makeInOutDcomplex)
  return ArrayTest.ArrayOps.createDcomplex(len)
# DO-NOT-DELETE splicer.end(makeInOutDcomplex)

def makeInOutFcomplex(a, len):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<fcomplex> a
  # int len
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<fcomplex> a
  #

# DO-NOT-DELETE splicer.begin(makeInOutFcomplex)
  return ArrayTest.ArrayOps.createFcomplex(len)
# DO-NOT-DELETE splicer.end(makeInOutFcomplex)

def makeInOut2Int(a, d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,2> a
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,2> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut2Int)
  return ArrayTest.ArrayOps.create2Int(d1,d2)
# DO-NOT-DELETE splicer.end(makeInOut2Int)

def makeInOut2Double(a, d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double,2> a
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double,2> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut2Double)
  return ArrayTest.ArrayOps.create2Double(d1,d2)
# DO-NOT-DELETE splicer.end(makeInOut2Double)

def makeInOut2Float(a, d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<float,2> a
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<float,2> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut2Float)
  return ArrayTest.ArrayOps.create2Float(d1,d2)
# DO-NOT-DELETE splicer.end(makeInOut2Float)

def makeInOut2Dcomplex(a, d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<dcomplex,2> a
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<dcomplex,2> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut2Dcomplex)
  return ArrayTest.ArrayOps.create2Dcomplex(d1,d2)
# DO-NOT-DELETE splicer.end(makeInOut2Dcomplex)

def makeInOut2Fcomplex(a, d1, d2):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<fcomplex,2> a
  # int d1
  # int d2
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<fcomplex,2> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut2Fcomplex)
  return ArrayTest.ArrayOps.create2Fcomplex(d1,d2)
# DO-NOT-DELETE splicer.end(makeInOut2Fcomplex)

def makeInOut3Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,3> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,3> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut3Int)
  return ArrayTest.ArrayOps.create3Int()
# DO-NOT-DELETE splicer.end(makeInOut3Int)

def makeInOut4Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,4> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,4> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut4Int)
  return ArrayTest.ArrayOps.create4Int()
# DO-NOT-DELETE splicer.end(makeInOut4Int)

def makeInOut5Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,5> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,5> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut5Int)
  return ArrayTest.ArrayOps.create5Int()
# DO-NOT-DELETE splicer.end(makeInOut5Int)

def makeInOut6Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,6> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,6> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut6Int)
  return ArrayTest.ArrayOps.create6Int()
# DO-NOT-DELETE splicer.end(makeInOut6Int)

def makeInOut7Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<int,7> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<int,7> a
  #

# DO-NOT-DELETE splicer.begin(makeInOut7Int)
  return ArrayTest.ArrayOps.create7Int()
# DO-NOT-DELETE splicer.end(makeInOut7Int)

def checkGeneric(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (dmn, tp)
  # int dmn
  # int tp
  #

  """\
Return as out parameters the type and dimension of the 
array passed in. If a is NULL, dimen == type == 0 on exit.
The contents of the array have the default values for a 
newly created array.
"""
# DO-NOT-DELETE splicer.begin(checkGeneric)
  # Put your code here...

  if(isinstance(a,NoneType)):
      return (0,0)

  dmn = len(a.shape);
  tp = numTypeToSidlType(typecode(a));  #Take a guess at the correct type!
  
  return (dmn,tp)

# DO-NOT-DELETE splicer.end(checkGeneric)

def createGeneric(dmn, tp):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int dmn
  # int tp
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<> _return
  #

  """\
Create an array of the type and dimension specified and
return it. A type of 0 causes a NULL array to be returned.
"""
# DO-NOT-DELETE splicer.begin(createGeneric)
  # Put your code here...
  ashape = []; #the shape of the array, made up from the dimension
  if(dmn < 1 or tp < 1):
      return None;
  
  for i in range(dmn):
      ashape.append(3);

  return createArrayByType(tp, ashape);

# DO-NOT-DELETE splicer.end(createGeneric)

def passGeneric(inArg, inOutArg):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<> inArg
  # array<> inOutArg
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, inOutArg, outArg)
  # array<> _return
  # array<> inOutArg
  # array<> outArg
  #

  """\
Testing passing generic arrays using every possible mode.
The returned array is a copy of inArg, so if inArg != NULL,
the return value should != NULL. outArg is also a copy of
inArg.
If inOutArg is NULL on entry, a 2-D array of int that should
pass check2Int is returned.
If inOutArg is not NULL on entry and its dimension is even,
it is returned unchanged; otherwise, NULL is returned.
"""
# DO-NOT-DELETE splicer.begin(passGeneric)
  # Put your code here...
  ret = None;
  outArg = None;
  if (not isinstance(inArg,NoneType)):
      ret = inArg.copy();
      outArg = inArg.copy();

  if(not isinstance(inOutArg,NoneType)):
      if(isOdd(len(inOutArg.shape))):
          inOutArg = None;
  else:
      inOutArg = ArrayTest.ArrayOps.create2Int(3,3);
                
  return (ret,inOutArg, outArg);
          
  
# DO-NOT-DELETE splicer.end(passGeneric)

def initRarray1Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<int> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # rarray<int> a
  #

# DO-NOT-DELETE splicer.begin(initRarray1Int)
  # Put your code here...

  if (a.shape[0] >= 0):
      prime = 0
      for i in range(a.shape[0]):
          prime = nextPrime(prime)
          a[i] = prime
                
  return a  
# DO-NOT-DELETE splicer.end(initRarray1Int)

def initRarray3Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<int,3> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # rarray<int,3> a
  #

# DO-NOT-DELETE splicer.begin(initRarray3Int)
  # Put your code here...
  for i in range(a.shape[0]):
    for j in range(a.shape[1]):
      for k in range(a.shape[2]):
        a[i,j,k] = arrayValue((i,j,k))
        
  return a
 
# DO-NOT-DELETE splicer.end(initRarray3Int)

def initRarray7Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<int,7> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # rarray<int,7> a
  #

# DO-NOT-DELETE splicer.begin(initRarray7Int)
  # Put your code here...
  for i in range(a.shape[0]):
    for j in range(a.shape[1]):
      for k in range(a.shape[2]):
        for l in range(a.shape[3]):
          for m in range(a.shape[4]):
            for n in range(a.shape[5]):
              for o in range(a.shape[6]):
                a[i,j,k,l,m,n,o] = arrayValue((i,j,k,l,m,n,o))
  return a
# DO-NOT-DELETE splicer.end(initRarray7Int)

def initRarray1Double(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<double> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # rarray<double> a
  #

# DO-NOT-DELETE splicer.begin(initRarray1Double)
  # Put your code here...
  if (a.shape[0] >= 0):
      for i in range(a.shape[0]):
          a[i] = powTwo(-i)
  return a          
# DO-NOT-DELETE splicer.end(initRarray1Double)

def initRarray1Dcomplex(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<dcomplex> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # rarray<dcomplex> a
  #

# DO-NOT-DELETE splicer.begin(initRarray1Dcomplex)
  # Put your code here...
  if (a.shape[0] >= 0):
      for i in range(a.shape[0]):
          a[i] = powTwo(i) + powTwo(-i)*1j
  return a
# DO-NOT-DELETE splicer.end(initRarray1Dcomplex)

def checkRarray1Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<int> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkRarray1Int)
  # Put your code here...
  return checkInt(a)
# DO-NOT-DELETE splicer.end(checkRarray1Int)

def checkRarray3Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<int,3> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkRarray3Int)
  # Put your code here...
  return check3Int(a)
# DO-NOT-DELETE splicer.end(checkRarray3Int)

def checkRarray7Int(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<int,7> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkRarray7Int)
  # Put your code here...
  return check7Int(a)
# DO-NOT-DELETE splicer.end(checkRarray7Int)

def checkRarray1Double(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<double> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkRarray1Double)
  # Put your code here...
  return checkDouble(a)
# DO-NOT-DELETE splicer.end(checkRarray1Double)

def checkRarray1Dcomplex(a):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<dcomplex> a
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkRarray1Dcomplex)
  # Put your code here...
  return checkDcomplex(a)
# DO-NOT-DELETE splicer.end(checkRarray1Dcomplex)

def matrixMultiply(a, b, res):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<int,2> a
  # rarray<int,2> b
  # rarray<int,2> res
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # rarray<int,2> res
  #

# DO-NOT-DELETE splicer.begin(matrixMultiply)
  # Put your code here...
  locMatrixMultiply(a,b,res)
  return res
# DO-NOT-DELETE splicer.end(matrixMultiply)

def checkMatrixMultiply(a, b, res):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # rarray<int,2> a
  # rarray<int,2> b
  # rarray<int,2> res
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

# DO-NOT-DELETE splicer.begin(checkMatrixMultiply)
  # Put your code here...
  test = zeros((a.shape[0], b.shape[1], ), int32)
  savespace(test)
  locMatrixMultiply(a,b,test);
  for i in range(a.shape[0]):
    for j in range(b.shape[1]):
      if (test[i,j] != res[i,j]):
        return 0
  return 1
# DO-NOT-DELETE splicer.end(checkMatrixMultiply)

# DO-NOT-DELETE splicer.begin(_before_type)
# Put your code here...
# DO-NOT-DELETE splicer.end(_before_type)

class ArrayOps:

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = ArrayTest.ArrayOps.ArrayOps(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def mm(self, a, b, res):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # rarray<int,2> a
    # rarray<int,2> b
    # rarray<int,2> res
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # rarray<int,2> res
    #

# DO-NOT-DELETE splicer.begin(mm)
    # Insert-Code-Here {mm} ()
    pass
# DO-NOT-DELETE splicer.end(mm)

  def checkmm(self, a, b, res):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # rarray<int,2> a
    # rarray<int,2> b
    # rarray<int,2> res
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # bool _return
    #

# DO-NOT-DELETE splicer.begin(checkmm)
    # Insert-Code-Here {checkmm} ()
    pass
# DO-NOT-DELETE splicer.end(checkmm)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# File:          G_Impl.py
# Symbol:        Inherit.G-v1.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class Inherit.G in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


# DO-NOT-DELETE splicer.begin(_initial)
print "Hello"
# DO-NOT-DELETE splicer.end(_initial)

import Inherit.A
import Inherit.D
import Inherit.G
import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
# Put your code here...
# DO-NOT-DELETE splicer.end(_before_type)

class G:

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = Inherit.G.G(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def g(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

# DO-NOT-DELETE splicer.begin(g)
    return "G.g"
# DO-NOT-DELETE splicer.end(g)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# File:          __init__.py
# Symbol:        sidl.io-v0.9.17
# Symbol Type:   package
# Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
# Release:       $Name:  $
# Revision:      @(#) $Id: $
# Description:   package initialization code
# 
# Copyright (c) 2000-2002, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by the Components Team <components@llnl.gov>
# All rights reserved.
# 
# This file is part of Babel. For more information, see
# http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
# for Our Notice and the LICENSE file for the GNU Lesser General Public
# License.
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by
# the Free Software Foundation) version 2.1 dated February 1999.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
# conditions of the GNU Lesser General Public License for more details.
# 
# You should have recieved a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# 
# WARNING: Automatically generated; changes will be lost
# 
#


""" 
This package has some I/O capability that's not core to the 
SIDL object model, but still needed by parts of the generated code
"""

__all__ = [
   "Deserializer",
   "IOException",
   "Serializable",
   "Serializer" ]

try:
  from pkgutil import extend_path
  __path__ = extend_path(__path__, __name__)
except: # ignore all exceptions
  pass



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

import string
import sys
import getopt
import os
import socket # to get hostname

import gantlet
from gantlet.session_archiver import session_archiver, EntrySort
from email_display import email_display, rfc822time
from sys import maxint
from time import strptime, mktime, time

html_begin='''
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
  <head>
    <title>Gantlet Test Results</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
  </head>
  <body>
'''

col_head = {
    'date':'<th width="100" rowspan="2">date</th>',
    'profile':'<th width="150" rowspan="2">profile</th>',
    'session':'<th width="100" rowspan="2">session</th>',
    'package':'<th width="100" rowspan="2">package</th>',
    'time':'<th width="100" rowspan="2">time</th>',
    'path':'<th rowspan="2">path</th>'}

table_begin='''
    <table border="3">
      <tr>
        %s
        <th colspan="5">Tests</th>
        <th colspan="4">Parts</th>
      </tr>
      <tr>
        <th width="50">Total</th>
        <th width="50">Pass</th>
        <th width="50">Xfail</th>
        <th width="50">Fail</th>
        <th width="50">Broken</th>
        <th width="50">Total</th>
        <th width="50">Pass</th>
        <th width="50">Xfail</th>
        <th width="50">Fail</th>
      </tr>      
'''

table_end="""
    </table><hr>
"""

html_end="""
    <address>Generated By Gantlet %s<br>
             Gary Kumfert, Ph.D. &lt;<a href=\"mailto:kumfert@llnl.gov\">kumfert@llnl.gov</a>&gt;<br>
             Lawrence Livermore National Laboratory <a href=\"http://www.llnl.gov\">http://www.llnl.gov</a>
    </address>  
  </body>
</html>
"""%gantlet.VERSION

color = { 'none':'',
          'green':' bgcolor="#aaffaa"',
          'yellow':' bgcolor="#ffffaa"',
          'red':' bgcolor="#ffaaaa"'}
data = [ 'total_tests', 'passed_tests', 'xfailed_tests', 'failed_tests', \
         'broken_tests', 'total_parts', 'passed_parts', 'xfailed_parts', \
         'failed_parts' ]

class html_viewer:
    def __init__( self, sort_order=['profile','session','date'],
                  break_tables_on='package', display_order=['date','profile','session'] ):
        self.rev_sort_order = []
        self.rev_sort_order.extend(sort_order)
        self.rev_sort_order.reverse()
        self.break_tables_on = break_tables_on
        self.display_order = display_order

    def sort_msgs(self, msgs ):
        foo = None
        for key in self.rev_sort_order:
            foo = EntrySort( key, foo )        
        msgs.sort( foo )
        return msgs
        
    def html_page(self, f, msgs):
        """write an HTML page"""
        f.write(html_begin)
        table = {}
        for msg in msgs:
            if not table.has_key( msg[ self.break_tables_on ]):
                table[ msg[ self.break_tables_on ] ] = []
            table[ msg[ self.break_tables_on ]].append(msg)

        msgnames = table.keys()
        msgnames.sort()
        for msgname in msgnames:
            msgs = self.sort_msgs( table[ msgname ] )            
            self.html_table( f, msgname, msgs )
        f.write(html_end)
                      
    def html_table( self, f, tablehead, msgs ):
        """ write an HTML table"""
        
        f.write("  <h1>%s test results</h1>\n" % tablehead )
        str = ""
        for cat in self.display_order:
            if col_head.has_key( cat ):
                str = str + col_head[ cat ]
            else:
                str = str + '<th width="100" rowspan="2">' + cat + '</th>'
        f.write(table_begin%str)
        for msg in msgs:
            f.write(self.html_line( f, msg ))
        f.write(table_end)
    
    def html_line( self, f, msg ):
        colormap = { 'total_tests':color['none'], \
                     'passed_tests':color['green'], \
                     'xfailed_tests':color['yellow'], \
                     'failed_tests':color['red'], \
                     'broken_tests':color['red'], \
                     'total_parts':color['none'], \
                     'passed_parts':color['green'], \
                     'xfailed_parts':color['yellow'], \
                     'failed_parts':color['red'] }
        front=[]
        cells=[]
        sum = 0
        for cat in self.display_order:
            if cat == 'path':
                front.append('<td><a href="%s">[File]</a></td>'%msg[cat])
            else:
                front.append("<td>%s</td>"%msg[cat])
        for test in data:
            i=int(msg[test])    
            if i>0:
                cells.append('<td align="right"%s>%d</td>'%(colormap[test],i) )
            else:
                cells.append('<td align="right">0</td>')
            sum = sum+i
        if sum==0:
            cells=[]
            cells.append("<th colspan=9%s>BROKEN</th>" % color['red'] )
        cells = front + cells
        line = "      <tr>\n        %s      </tr>\n" % string.join(cells,'\n        ')
        return line
    

help_string="""
HELP

 -h   | --help                       Display this message
 -v   | --verbose                    Verbose
 -r<> | --repository=<>              <>=string, path to where Gantlet sessions
                                      are stored (default=\".\")
 -o<> | --output=<>                  <>=string, path to where html is generated
                                      (default=\"index.html\"
 -e<> | --email=<>                   <>=string, send email to this address
 -m<> | --mailserver=<>              <>=string, domain name of SMTP server
 -d<> | --display-order=<>           <>=comma separated list of attributes
 -s<> | --sort-order=<>              <>=comma separated list of attributes
 -l<> | --select=<>                  <>=attributes (see below)
 -b<> | --break-tables-on=<>         <>=attribute
 -t   | --today                      select only todays tests

 attributes = { time, package, session, profile, date,path }
 NOTE: --select works by regex on package, profile, session, and
         date attributes.  Default is
         --select=package=.*,profile=.*,session=.*,date=.*
 """

shortopts="hvr:o:d:s:e:m:b:tl:"
longopts=["help","verbose","repository","output","display-order","sort-order","email","mailserver","break-tables-on","today","select"]

std_header = 'From: %s\nTo: %s\nDate: %s\nSubject: %s\nMIME-Version: 1.0\nContent-Type: text/html; charset=iso-8859-1\n'

def parse_date(datestr):
    if (datestr):
        if (datestr == "24 hours"):
            now = time()
            return (now - 24*60*60, now)
        else:
            tm = strptime(datestr,"%Y-%m-%d")
            if (tm):
                t = mktime(tm)
                return (t, t + 24*60*60)
    return (0, maxint)

def _test():
    verbose=0
    repository = "."
    output = "index.html"
    display=['session','profile','hostname','date','time','path']
    sort=['session','profile']
    breakon='package'
    email=None
    mailserver=None
    selectdate=None
    selectpackage=".*"
    selectprofile=".*"
    selectsession=".*"
    selects=[]
    (opts,args)=getopt.getopt(sys.argv[1:],shortopts,longopts)
    for (o,v) in opts:
        if o in ("-h","--help"):
            print help_string
            sys.exit(0)
        elif o in ("-v", "--verbose"):
            verbose=1
        elif o in ("-r","--repository"):
            repository=v
        elif o in ("-o","--output"):
            output=v
        elif o in ("-d","--display"):
            display=v.split(',')
        elif o in ("-s","--sort-order"):
            sort=v.split(',')
        elif o in ("-b","--break-tables-on"):
            breakon=v
        elif o in ("-e","--email"):
            email=v
        elif o in ("-m","--mailserver"):
            mailserver=v
        elif o in ("-b","--break-tables-on"):
            breakon=v
        elif o in ("-t","--today"):
            selectdate="24 hours"
        elif o in ("-l","--select"):
            selects = v.split(',')
            for s in selects:
                (x,y)=s.split('=')
                if x == "package":
                    selectpackage=y
                elif x == "profile":
                    selectprofile=y
                elif x == "session":
                    selectsession=y
                elif x == "date":
                    selectdate=y
                else:
                    print "Unrecognized select option:", x

    mgr = session_archiver( repository )
    daterange = parse_date(selectdate)
    msgs = mgr.select(package=selectpackage,profile=selectprofile,
                      session=selectsession,
                      starttime=daterange[0], endtime=daterange[1])
    f = open(output,'w')
    viewer = html_viewer(break_tables_on=breakon,
                         display_order=display,
                         sort_order=sort
                         )
    viewer.html_page( f, msgs )
    f.close()
    
    if not( email == None or mailserver == None):
        f = open(output,'r')
        e = email_display( None, email, mailserver)
        e.file = f
        e.verbose = verbose
        whoami = 'unknown'
        try:
            whoami = os.getlogin()
        except:
            pass
        hostname = socket.getfqdn()
        email_from = whoami + "@" + hostname
        date = rfc822time()
        subject="Gantlet Results"
        header=std_header %( email_from, email, date, subject )
        e.email_from = email_from
        e.email_to = email
        e.email_file_via_SMTP(header)

    f.close()

if __name__ == '__main__': _test()




#
# File:          SortTest_Impl.py
# Symbol:        sort.SortTest-v0.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class sort.SortTest in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""Run a bunch of sorts through a stress test.
"""

# DO-NOT-DELETE splicer.begin(_initial)
import sys
# DO-NOT-DELETE splicer.end(_initial)

import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sort.SortTest
import sort.SortingAlgorithm
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_static)
import sort.Container
import sort.CompInt
import sort.Comparator
import sort.IntegerContainer
from synch import RegOut

testSizes = [ 0, 1, 2, 3, 4, 7, 10, 51, 64 ]

def notSorted(cont, comp):
  size = cont.getLength()
  for i in xrange(1, size):
    if (cont.compare(i-1, i, comp) > 0):
      return 1
  return 0

def sortAndReport(alg, cont, comp, result):
  tracker = RegOut.getInstance()
  alg.reset()
  alg.sort(cont, comp)
  swpCnt = alg.getSwapCounter()
  cmpCnt = alg.getCompareCounter()
  tracker.writeComment("compares (" +  str(cmpCnt.getCount()) +
                 ") swaps (" + str(swpCnt.getCount()) + ")")
  if (notSorted(cont, comp)):
    
    tracker.writeComment("sort failed!!")
    result = 0
  return result
# DO-NOT-DELETE splicer.end(_before_static)

def stressTest(algs):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<sort.SortingAlgorithm> algs
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

  """\
Perform the array stress test.

Return true if all the algorithms work okay.
"""
# DO-NOT-DELETE splicer.begin(stressTest)
  result = 1
  tracker = RegOut.getInstance()
  for alg in algs:
    data = sort.IntegerContainer.IntegerContainer()
    cont = sort.Container.Container(data)
    intcomp = sort.CompInt.CompInt()
    comp = sort.Comparator.Comparator(intcomp)
    name = alg.getName()
    tracker.writeComment("****ALGORITHM IS " +  name + "****")
    for size in testSizes:
      intcomp.setSortIncreasing(1)
      tracker.writeComment("DATA SIZE " + str(size))
      data.setLength(size)
      result = sortAndReport(alg, cont, comp, result)
      tracker.writeComment("pre-sorted list")
      result = sortAndReport(alg, cont, comp, result)
      tracker.writeComment("reverse sorted list")
      intcomp.setSortIncreasing(0)
      result = sortAndReport(alg, cont, comp, result)
  sys.stdout.flush()
  return result
# DO-NOT-DELETE splicer.end(stressTest)

# DO-NOT-DELETE splicer.begin(_before_type)
# Put your code here...
# DO-NOT-DELETE splicer.end(_before_type)

class SortTest:
  """\
Run a bunch of sorts through a stress test.
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = sort.SortTest.SortTest(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# File:          IntegerContainer_Impl.py
# Symbol:        sort.IntegerContainer-v0.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class sort.IntegerContainer in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""Integer container.
"""

# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sort.Comparator
import sort.Container
import sort.IntegerContainer
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
import sidlPyArrays
if sidlPyArrays.type == "numpy":
  from numpy import zeros, object
  PyObject = object
elif sidlPyArrays.type == "numeric":
  from Numeric import zeros, PyObject
import sort.Integer
import random
from synch import RegOut
from types import NoneType

try:
  # available in 2.0 and later
  shuffle = random.shuffle
except AttributeError:
  def shuffle(x):
    for i in xrange(len(x)-1, 0, -1):
      j = random.randint(0, i)
      x[i], x[j] = x[j], x[i]
# DO-NOT-DELETE splicer.end(_before_type)

class IntegerContainer:
  """\
Integer container.
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = sort.IntegerContainer.IntegerContainer(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    self.d_elements = None
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def setLength(self, len):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # int len
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
This sets the container length and pseudo-randomly orders the
Integer elements contained.
"""
# DO-NOT-DELETE splicer.begin(setLength)
    # create a 1-d Numeric Python array with size objects
    self.d_elements = zeros((len,), PyObject)
    for i in xrange(len):
      self.d_elements[i] = sort.Integer.Integer()
      self.d_elements[i].setValue(i+1)
    shuffle(self.d_elements)
# DO-NOT-DELETE splicer.end(setLength)

  def getLength(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # int _return
    #

    """\
Return the number of elements in the container.
"""
# DO-NOT-DELETE splicer.begin(getLength)
    if not isinstance(self.d_elements,NoneType):
      return self.d_elements.shape[0]
    else:
      return 0
# DO-NOT-DELETE splicer.end(getLength)

  def compare(self, i, j, comp):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # int i
    # int j
    # sort.Comparator comp
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # int _return
    #

    """\
Return -1 if element i is less than element j, 0 if element i
is equal to element j, or otherwise 1.
"""
# DO-NOT-DELETE splicer.begin(compare)
    size = self.__IORself.getLength()
    result = 0
    if ((0 <= i < size) and (0 <= j < size)):
      i1 = sidl.BaseInterface.BaseInterface(self.d_elements[i])
      i2 = sidl.BaseInterface.BaseInterface(self.d_elements[j])
      result = comp.compare(i1, i2)
    return result
# DO-NOT-DELETE splicer.end(compare)

  def swap(self, i, j):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # int i
    # int j
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Swap elements i and j.
"""
# DO-NOT-DELETE splicer.begin(swap)
    size = self.__IORself.getLength()
    if ((0 <= i < size) and (0 <= j < size)):
      self.d_elements[i], self.d_elements[j] = \
           self.d_elements[j], self.d_elements[i]
# DO-NOT-DELETE splicer.end(swap)

  def output(self, s, e):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # int s
    # int e
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Print elements s through e-1
"""
# DO-NOT-DELETE splicer.begin(output)
    size = self.__IORself.getLength()
    listcontents = "list"
    for i in xrange(size):
      listcontents = listcontents + " " + str(self.d_elements[i])
    RegOut.getInstance().writeComment(listcontents);
# DO-NOT-DELETE splicer.end(output)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

import re
import rfc822
import string
import StringIO
import xml.sax
import xml.sax.handler

class SAXShortCircuitException( xml.sax.SAXException ):
    """Exception used to keep from parsing the entire document(only need header)"""
    pass

class session_email_parser ( xml.sax.handler.ContentHandler ) :
    """A XML-encoded Gantlet report recieved via email"""

    def __init__( self, f, id=0, content='' ):
        """Initialize from an email message encoded as a single long string
        
        f - the entire email file as one big string, hopefully containing xml
        id  - a unique id number (email message id)
        """
        #        self.msg = msg
        if content:
            self.msg = content
        else:
            self.msg = string.join(f.readlines(),"")
            f = StringIO.StringIO(self.msg)
        self.id = id
        hdr = rfc822.Message(f)  # parse mail headers into dict
        if not hdr.has_key('Subject'):
            self.is_gantlet = None
            self.subject='(no subject)'
            return
        self.subject = hdr['Subject']
        if (self.subject[:13] != "<gantlet-xml " and
	   self.subject[:14] != "<gauntlet-xml "):  #backward compatable
            self.is_gantlet = None
            return
        self.fromwhom = hdr['From']
        self.is_gantlet = 1
        self.date = hdr['Date']
        self.date_tuple = hdr.getdate('Date')
        self.fp = hdr.fp
        self.attr = {}
        self.attr['package'] = ''
        self.attr['profile'] = ''
        self.attr['session'] = ''
        self.attr['elapsedtime'] = '0'
        self.attr['total_tests'] = '0'
        self.attr['passed_tests'] = '0'
        self.attr['xfailed_tests'] = '0'
        self.attr['failed_tests' ] = '0'
        self.attr['broken_tests' ] = '0'
        self.attr['total_parts' ] = '0'
        self.attr['passed_parts' ] = '0'
        self.attr['xfailed_parts' ] = '0'
        self.attr['failed_parts' ] = '0'
        self.PVT_parse_attr()
        self.PVT_clean_date()

    def startElement( self, name, attrs ):
        """Overrides xml.sax.handler.ContentHandler.startElement()"""
        if name == 'attribute':
            self.attr[ str( attrs['key'] )  ] = str( attrs['value'] )

    def endElement( self, name ):
        """Overrides xml.sax.handler.ContentHandler.startElement()"""
        if name == 'head':
            raise SAXShortCircuitException('no more attributes here')

    def PVT_parse_attr( self ):
        """PRIVATE: Load attributes at front of XML file using SAX"""
        if re.search('result="BROKEN"', self.subject):
            self.PVT_infer_broken_attrs()
        else:
            self.attr['result'] = re.search('result="([^"]*)"',
                                            self.subject ).group(1)
            tparser = xml.sax.make_parser()
            tparser.setContentHandler( self )
            try:
                tparser.parse( self.fp )
            except SAXShortCircuitException:
                # exception raised to stop before parsing whole document
                pass

    def PVT_clean_date(self):
        """PRIVATE: convert 'Thu, 18 Apr 2002' to '2002-04-18'"""
        date = str(self.attr['date']) + ' 00:00:00'
        s = rfc822.parsedate( date )
        if s:
            self.attr['date_text'] = self.attr['date']
            self.attr['date'] = '%04d-%02d-%02d' % s[0:3]
            
    def PVT_infer_broken_attrs( self ):
        """PRIVATE: Infer needed attributes from email header"""
        self.attr['package'] = re.search('package="([^"]*)"', self.subject ).group(1)
        self.attr['profile'] = re.search('profile="([^"]*)"', self.subject ).group(1)
        self.attr['session'] = re.search('session="([^"]*)"', self.subject ).group(1)
        self.attr['date_text'] = self.date
        self.attr['date'] = "%04d-%02d-%02d" % self.date_tuple[0:3]
        self.attr['time'] = "%d:%d:%d" % self.date_tuple[3:6]
        email = rfc822.parseaddr( self.fromwhom )[1]
        self.attr['whoami'] = re.search('^([^@]*)@', email ).group(1)
        self.attr['hostname'] = re.search('@(.*)', email ).group(1)        
        self.attr['result'] = 'BROKEN'



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.
import re
import gantlet.events
from gantlet.result_base import result_base, Part

class xml_result( result_base ):
    """
    This class encapsulates a single XML encoded test
    """
    def __init__( self, invocation, scoreboard ):
        self.invocation = invocation
        result_base.__init__(self, invocation.symbolic_name, scoreboard, "xml_result" )
        self.n_parts = 0
        self.scoreboard = scoreboard

    def test_begin( self, attrs ):
        self.number = self.scoreboard.get_id()
        attrname = attrs['name']
        if self.invocation.filename == self.name: # if name not explicitly altered
            if self.invocation.basename == "index.xml":
                # strip 'index.xml' from symbolic name
                self.name = self.invocation.dirname + '/'        
        if len(self.name) < 1 :
            self.name = attrname           
        elif attrname[0] == '/' or self.name[-1] == '/':
            self.name = self.name + attrname
        else:
            self.name = "%s[%s]" % ( self.name, attrname )
        self.declared_result = attrs['result']
        self.result = self.declared_result  # assume its correct (for now)
        self.declared_nparts = int(attrs['nparts'])
        self.n_parts = self.declared_nparts # assume its correct (for now)
        self.type = attrs['type']
        self.part = []
        self.scoreboard.event['test_begin'].publish(
            gantlet.events.test_begin_event, self )

    def part_begin( self, attrs ):
        part = Part()
        part.number = int(attrs['number'])
        part.result = attrs['result']
        self.part.append(part)
        self.scoreboard.event['part_begin'].publish(
            gantlet.events.part_begin_event, ( self, part.number ) )

    def part_end( self ):
        number = self.part[-1].number
        self.scoreboard.event['part_end'].publish(
            gantlet.events.part_end_event, (self, number ) )

    def test_end( self ):
        self.PTC_confirm_nparts()
        self.PTC_confirm_result()
        self.scoreboard.event['test_end'].publish(
            gantlet.events.test_end_event, self )

    def warning_begin( self, attrs ):
        self.canon = attrs['canon']
            
    def warning_end( self, msg ):
        self.warning( msg, self.canon )
        self.canon = ''

    def error_begin( self, attrs ):
        self.canon = attrs['canon']
        #match = re.search("exit_status=(\d*)", self.canon )
        #if match:
        #    self.exit_status = int( match.group(1) )
        
    def error_end( self, msg ):
        self.broken( msg, self.canon )
        self.canon = ''

    def comment_begin( self, attrs ):
        pass

    def comment_end( self, msg ):
        if len( msg ):
            self.scoreboard.event['comment'].publish(
                gantlet.events.comment_event, msg )

    def PVT_start_stub( self, attrs ):
        pass

    def PVT_end_stub( self ):
        pass



